#include <stdio.h>
#include "LibAard.h"
#include "Commands.h"
#include "AardRemoteMessage.h"
#include "RawIR.h"

int
main(int argc, char ** argv) {
   AardRemoteMessage * msg;

   if (argc != 2) {
      fprintf(stderr, "usage: testaard devicename\n");
      return 2;
   }
   LibAard l;
   if (l.OpenAardRemote(argv[1]) < 0) {
      fprintf(stderr, "open of %s failed\n", argv[1]);
      return 2;
   }

   // Get synced up and make sure the aardremote is talking.

   int bilgecount = 0;
   while (bilgecount < 2) {
      int code = l.WaitForStartup();
      if (code == MKAS_RESET) {
         printf("WaitForStartup returned due to RESET detected\n");
         bilgecount = 0;
      } else if (code == MKAS_BILGE) {
         printf("WaitForStartup returned due to BILGE detected\n");
         ++bilgecount;
      } else {
         printf("WaitForStartup returned unexpected code 0x%x", code);
         bilgecount = 0;
      }
   }

   // Now query everything that can be queried.

   l.SendCommand(MKCR_QMICROS);
   msg = l.WaitMessage(100);
   if (msg == NULL) {
      printf("No answer to MKCR_QMICROS query.\n");
   } else {
      if ((msg->MessageKind() == MK_REPLY) &&
          (msg->FunctionCode() == MKCR_QMICROS) &&
          (msg->MessageLength() == 3)) {
         printf("AardRemote microsecond clock is 0x%04x%04x\n",
                msg->MessageBody()[2], msg->MessageBody()[1]);
      } else {
         printf("Unexpected message from microsecond query\n");
      }
   }
   delete msg;
   l.SendCommand(MKCR_QVERSION);
   msg = l.WaitMessage(100);
   if (msg == NULL) {
      printf("No answer to MKCR_QVERSION query.\n");
   } else {
      if ((msg->MessageKind() == MK_REPLY) &&
          (msg->FunctionCode() == MKCR_QVERSION) &&
          (msg->MessageLength() == 2)) {
         printf("AardRemote microcode version is %u\n", msg->MessageBody()[1]);
      } else {
         printf("Unexpected message from version query\n");
      }
   }
   delete msg;
   l.SendCommand(MKCR_QRECEIVER);
   msg = l.WaitMessage(100);
   if (msg == NULL) {
      printf("No answer to MKCR_QRECEIVER query.\n");
   } else {
      if ((msg->MessageKind() == MK_REPLY) &&
          (msg->FunctionCode() == MKCR_QRECEIVER) &&
          (msg->MessageLength() == 2)) {
         printf("AardRemote receiver is %s\n", msg->MessageBody()[1] ? "on" : "off");
      } else {
         printf("Unexpected message from receiver query\n");
      }
   }
   delete msg;
   l.SendCommand(MKCR_QFREQUENCY);
   msg = l.WaitMessage(100);
   if (msg == NULL) {
      printf("No answer to MKCR_QFREQUENCY query.\n");
   } else {
      if ((msg->MessageKind() == MK_REPLY) &&
          (msg->FunctionCode() == MKCR_QFREQUENCY) &&
          (msg->MessageLength() == 2)) {
         printf("AardRemote sender set to %ukHz\n", msg->MessageBody()[1]);
      } else {
         printf("Unexpected message from frequency query\n");
      }
   }
   delete msg;
   l.SendCommand(MKCR_QUEUESIZE);
   msg = l.WaitMessage(100);
   if (msg == NULL) {
      printf("No answer to MKCR_QUEUESIZE query.\n");
   } else {
      if ((msg->MessageKind() == MK_REPLY) &&
          (msg->FunctionCode() == MKCR_QUEUESIZE) &&
          (msg->MessageLength() == 2)) {
         printf("AardRemote IR buffer size is %u\n", msg->MessageBody()[1]);
      } else {
         printf("Unexpected message from queue size query\n");
      }
   }
   delete msg;
   l.SendCommand(MKCR_QMICROS);
   msg = l.WaitMessage(100);
   if (msg == NULL) {
      printf("No answer to MKCR_QMICROS query.\n");
   } else {
      if ((msg->MessageKind() == MK_REPLY) &&
          (msg->FunctionCode() == MKCR_QMICROS) &&
          (msg->MessageLength() == 3)) {
         printf("AardRemote microsecond clock is 0x%04x%04x\n",
                msg->MessageBody()[2], msg->MessageBody()[1]);
      } else {
         printf("Unexpected message from microsecond query\n");
      }
   }
   delete msg;
   l.SendCommand(MKCR_RECEIVER, 1);
   msg = l.WaitMessage(100);
   if (msg == NULL) {
      printf("No answer to MKCR_RECEIVER command.\n");
   } else {
      if ((msg->MessageKind() == MK_REPLY) &&
          (msg->FunctionCode() == MKCR_RECEIVER) &&
          (msg->MessageLength() == 1)) {
         printf("AardRemote IR receiver turned ON, press a remote button.\n");
      } else {
         printf("Unexpected message from enable receiver command\n");
      }
   }
   delete msg;
   bool seen_end_of_ir = false;
   std::vector<unsigned short> irseq;
   bool goodseq = true;
   while (! seen_end_of_ir) {
      msg = l.WaitMessage(0);
      if ((msg->MessageKind() == MK_ASYNC) &&
          (msg->FunctionCode() == MKAS_RAWIR)) {
         const std::vector<unsigned short> & dat = msg->MessageBody();
         for (int i = 1; i < dat.size(); ++i) {
            unsigned short code = dat[i];
            if (BGIR_IS_STATUS(code)) {
               printf("status:");
               if (code & BGIR_END) {
                  printf(" end");
                  seen_end_of_ir = true;
               }
               if (code & BGIR_LOST_DATA) {
                  printf(" lost-data");
                  goodseq = false;
               }
               if (code & BGIR_LOST_TIME) {
                  printf(" lost-time");
                  goodseq = false;
               }
               if (code & BGIR_DISABLED) {
                  printf(" int-disabled");
               }
               printf("\n");
            } else {
               if (BGIR_IS_MARK(code)) {
                  irseq.push_back(code);
                  printf("M %u\n", BGIR_DURATION(code));
               } else if (BGIR_IS_SPACE(code)) {
                  irseq.push_back(code);
                  printf("S %u\n", BGIR_DURATION(code));
               } else {
                  printf("Unrecognized data: 0x%04x\n", code);
               }
            }
         }
      } else {
         printf("Unexpected message while waiting for IR data\n");
      }
      delete msg;
   }
   if ((irseq.size() > 0) && goodseq) {
      printf("OK, I'm about to send that same sequence...\n");
      usleep(1000000);
      printf("Here goes...\n");
      l.SendCommand(MKCR_SENDRAW,irseq);
      printf("I sent %u pulese to AardRemote\n", irseq.size());
      msg = l.WaitMessage(1000);
      if (msg == NULL) {
         printf("No answer to MKCR_SENDRAW command.\n");
      } else {
         if ((msg->MessageKind() == MK_REPLY) &&
             (msg->FunctionCode() == MKCR_SENDRAW) &&
             (msg->MessageLength() == 2)) {
            printf("AardRemote sent %u pulses\n", msg->MessageBody()[1]);
         } else {
            printf("Unexpected message from IR send command\n");
         }
      }
      delete msg;
   }
   return 0;
}
